@@ -5,6 +5,6 @@ confint.rma.glmm <- function(object, parm, level, digits, transf, targs, ...) {
    if (!inherits(object, "rma.glmm"))
       stop(mstyle$stop("Argument 'object' must be an object of class \"rma.glmm\"."))
 
-   stop(mstyle$stop("Method not available for objects of class \"rma.glmm\"."))
+   stop(mstyle$stop("Method not (yet) available for objects of class \"rma.glmm\"."))
 
 }
