@@ -18,6 +18,11 @@ rstudent.rma.peto <- function(model, digits, progbar=FALSE, ...) {
       digits <- .get.digits(digits=digits, xdigits=x$digits, dmiss=FALSE)
    }
 
+   ddd <- list(...)
+
+   if (.isTRUE(ddd$time))
+      time.start <- proc.time()
+
    #########################################################################
 
    delpred  <- rep(NA_real_, x$k.f)
@@ -72,6 +77,11 @@ rstudent.rma.peto <- function(model, digits, progbar=FALSE, ...) {
 
    out$digits <- digits
 
+   if (.isTRUE(ddd$time)) {
+      time.end <- proc.time()
+      .print.time(unname(time.end - time.start)[3])
+   }
+
    class(out) <- "list.rma"
    return(out)
 
