@@ -4,6 +4,8 @@
 
 context("Checking analysis example: morris2008")
 
+source("tolerances.r") # read in tolerances
+
 ### create datasets
 
 datT <- data.frame(
@@ -29,19 +31,19 @@ test_that("calculations of escalc() are correct for measure='SMCR'.", {
    datC <- escalc(measure="SMCR", m1i=m_post, m2i=m_pre, sd1i=sd_pre, ni=ni, ri=ri, data=datC)
 
    ### (results for this not given in paper)
-   expect_equivalent(round(datT$yi,4), c( 0.5056, 1.0481, 1.8054, 1.4181,  0.0801))
-   expect_equivalent(round(datT$vi,4), c( 0.0594, 0.0254, 0.2322, 0.1225,  0.0802))
-   expect_equivalent(round(datC$yi,4), c(-0.2365, 0.0958, 0.0000, 0.2667, -0.4250))
-   expect_equivalent(round(datC$vi,4), c( 0.0544, 0.0173, 0.0511, 0.0232,  0.0864))
+   expect_equivalent(datT$yi, c( 0.5056, 1.0481, 1.8054, 1.4181,  0.0801), tolerance=.tol[["est"]])
+   expect_equivalent(datT$vi, c( 0.0594, 0.0254, 0.2322, 0.1225,  0.0802), tolerance=.tol[["var"]])
+   expect_equivalent(datC$yi, c(-0.2365, 0.0958, 0.0000, 0.2667, -0.4250), tolerance=.tol[["est"]])
+   expect_equivalent(datC$vi, c( 0.0544, 0.0173, 0.0511, 0.0232,  0.0864), tolerance=.tol[["var"]])
 
    ### compute difference between treatment and control groups
    dat <- data.frame(yi = datT$yi - datC$yi, vi = datT$vi + datC$vi)
 
    ### compare with results on page 382 (Table 5)
-   expect_equivalent(round(dat$yi,2), c(0.74, 0.95, 1.81, 1.15, 0.51))
+   expect_equivalent(dat$yi, c(0.7421, 0.9524, 1.8054, 1.1514, 0.5050), tolerance=.tol[["est"]])
 
    ### (results for this not given in paper)
-   expect_equivalent(round(dat$vi,4), c(0.1138, 0.0426, 0.2833, 0.1458, 0.1667))
+   expect_equivalent(dat$vi, c(0.1138, 0.0426, 0.2833, 0.1458, 0.1667), tolerance=.tol[["var"]])
 
    ### use pooled pretest SDs
 
@@ -51,10 +53,10 @@ test_that("calculations of escalc() are correct for measure='SMCR'.", {
    dat$vi <- 2*(1-datT$ri) * (1/datT$ni + 1/datC$ni) + dat$yi^2 / (2*(datT$ni + datC$ni))
 
    ### compare with results on page 382 (Table 5)
-   expect_equivalent(round(dat$yi,2), c(0.77, 0.80, 1.20, 1.05, 0.44))
+   expect_equivalent(dat$yi, c(0.7684, 0.8010, 1.2045, 1.0476, 0.4389), tolerance=.tol[["est"]])
 
    ### (results for this not given in paper)
-   expect_equivalent(round(dat$vi,4), c(0.1134, 0.0350, 0.1425, 0.0681, 0.1634))
+   expect_equivalent(dat$vi, c(0.1134, 0.0350, 0.1425, 0.0681, 0.1634), tolerance=.tol[["var"]])
 
 })
 
@@ -65,16 +67,16 @@ test_that("calculations of escalc() are correct for measure='SMCC'.", {
    datC <- escalc(measure="SMCC", m1i=m_post, m2i=m_pre, sd1i=sd_post, sd2i=sd_pre, ni=ni, ri=ri, data=datC)
 
    ### (results for this not given in paper)
-   expect_equivalent(round(datT$yi,4), c( 0.5417, 1.0198, 2.6619, 1.9088,  0.0765))
-   expect_equivalent(round(datT$vi,4), c( 0.0573, 0.0304, 0.5048, 0.2822,  0.0716))
-   expect_equivalent(round(datC$yi,4), c(-0.2213, 0.1219, 0.0000, 0.5575, -0.2126))
-   expect_equivalent(round(datC$vi,4), c( 0.0512, 0.0240, 0.1111, 0.1050,  0.0730))
+   expect_equivalent(datT$yi, c( 0.5417, 1.0198, 2.6619, 1.9088,  0.0765), tolerance=.tol[["est"]])
+   expect_equivalent(datT$vi, c( 0.0573, 0.0304, 0.5048, 0.2822,  0.0716), tolerance=.tol[["var"]])
+   expect_equivalent(datC$yi, c(-0.2213, 0.1219, 0.0000, 0.5575, -0.2126), tolerance=.tol[["est"]])
+   expect_equivalent(datC$vi, c( 0.0512, 0.0240, 0.1111, 0.1050,  0.0730), tolerance=.tol[["var"]])
 
    ### compute difference between treatment and control groups
    dat <- data.frame(yi = datT$yi - datC$yi, vi = datT$vi + datC$vi)
 
    ### (results for this not given in paper)
-   expect_equivalent(round(dat$yi,2), c(0.76, 0.90, 2.66, 1.35, 0.29))
-   expect_equivalent(round(dat$vi,4), c(0.1086, 0.0544, 0.6159, 0.3872, 0.1447))
+   expect_equivalent(dat$yi, c(0.7630, 0.8979, 2.6619, 1.3513, 0.2891), tolerance=.tol[["est"]])
+   expect_equivalent(dat$vi, c(0.1086, 0.0544, 0.6159, 0.3872, 0.1447), tolerance=.tol[["var"]])
 
 })
