@@ -2,6 +2,8 @@
 
 context("Checking misc: confint() function")
 
+source("tolerances.r") # read in tolerances
+
 test_that("confint() works correctly for rma().", {
 
    data(dat.bcg, package="metafor")
@@ -9,10 +11,10 @@ test_that("confint() works correctly for rma().", {
    res <- rma(yi, vi, data=dat, method="DL")
    sav <- confint(res, fixed=TRUE, transf=exp)
 
-   expect_equivalent(round(c(sav$fixed), 4), c(0.4896, 0.3449, 0.6950))
-   expect_equivalent(round(sav$random[1,], 4), c(0.3088, 0.1197, 1.1115))
-   expect_equivalent(round(sav$random[3,], 4), c(92.1173, 81.9177, 97.6781))
-   expect_equivalent(round(sav$random[4,], 4), c(12.6861, 5.5303, 43.0680))
+   expect_equivalent(sav$fixed, c(0.4896, 0.3449, 0.6950), tolerance=.tol[["ci"]])
+   expect_equivalent(sav$random[1,], c(0.3088, 0.1197, 1.1115), tolerance=.tol[["var"]])
+   expect_equivalent(sav$random[3,], c(92.1173, 81.9177, 97.6781), tolerance=.tol[["het"]])
+   expect_equivalent(sav$random[4,], c(12.6861, 5.5303, 43.0680), tolerance=.tol[["het"]])
 
 })
 
@@ -22,7 +24,7 @@ test_that("confint() works correctly for rma.mh().", {
    res <- rma.mh(measure="RR", ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
    sav <- confint(res, transf=exp)
 
-   expect_equivalent(round(c(sav$fixed), 4), c(0.6353, 0.5881, 0.6862))
+   expect_equivalent(sav$fixed, c(0.6353, 0.5881, 0.6862), tolerance=.tol[["ci"]])
 
 })
 
@@ -32,6 +34,6 @@ test_that("confint() works correctly for rma.peto().", {
    res <- rma.peto(ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
    sav <- confint(res, transf=exp)
 
-   expect_equivalent(round(c(sav$fixed), 4), c(0.6222, 0.5746, 0.6738))
+   expect_equivalent(sav$fixed, c(0.6222, 0.5746, 0.6738), tolerance=.tol[["ci"]])
 
 })
